<?php
/**
 * Install and Deactivate Plugin Functions
 * @package KC_COM
 * @since WPAS 4.0
 */
if (!defined('ABSPATH')) exit;
if (!class_exists('Kc_Com_Install_Deactivate')):
	/**
	 * Kc_Com_Install_Deactivate Class
	 * @since WPAS 4.0
	 */
	class Kc_Com_Install_Deactivate {
		private $option_name;
		/**
		 * Hooks for install and deactivation and create options
		 * @since WPAS 4.0
		 */
		public function __construct() {
			$this->option_name = 'kc_com';
			$curr_version = get_option($this->option_name . '_version', 1);
			$new_version = constant(strtoupper($this->option_name) . '_VERSION');
			if (version_compare($curr_version, $new_version, '<')) {
				$this->set_options();
				$this->set_roles_caps();
				update_option($this->option_name . '_version', $new_version);
			}
			register_activation_hook(KC_COM_PLUGIN_FILE, array(
				$this,
				'install'
			));
			register_deactivation_hook(KC_COM_PLUGIN_FILE, array(
				$this,
				'deactivate'
			));
			add_action('wp_head', array(
				$this,
				'version_in_header'
			));
			add_action('admin_init', array(
				$this,
				'setup_pages'
			));
			add_action('admin_notices', array(
				$this,
				'install_notice'
			));
			add_action('admin_init', array(
				$this,
				'register_settings'
			) , 0);
			if (is_admin()) {
				$this->stax = new Emd_Single_Taxonomy('kc-com');
			}
			add_filter('tiny_mce_before_init', array(
				$this,
				'tinymce_fix'
			));
			add_action('init', array(
				$this,
				'init_extensions'
			) , 99);
		}
		public function version_in_header() {
			$version = constant(strtoupper($this->option_name) . '_VERSION');
			$name = constant(strtoupper($this->option_name) . '_NAME');
			echo '<meta name="generator" content="' . $name . ' v' . $version . ' - https://emdplugins.com" />' . "\n";
		}
		public function init_extensions() {
			do_action('emd_ext_init', $this->option_name);
		}
		/**
		 * Runs on plugin install to setup custom post types and taxonomies
		 * flushing rewrite rules, populates settings and options
		 * creates roles and assign capabilities
		 * @since WPAS 4.0
		 *
		 */
		public function install() {
			Emd_Panel::register();
			flush_rewrite_rules();
			$this->set_options();
			$this->set_roles_caps();
			do_action('emd_ext_install_hook', $this->option_name);
		}
		/**
		 * Runs on plugin deactivate to remove options, caps and roles
		 * flushing rewrite rules
		 * @since WPAS 4.0
		 *
		 */
		public function deactivate() {
			flush_rewrite_rules();
			$this->remove_caps_roles();
			$this->reset_options();
			do_action('emd_ext_deactivate', $this->option_name);
		}
		/**
		 * Register notification and/or license settings
		 * @since WPAS 4.0
		 *
		 */
		public function register_settings() {
			do_action('emd_ext_register', $this->option_name);
		}
		/**
		 * Sets caps and roles
		 *
		 * @since WPAS 4.0
		 *
		 */
		public function set_roles_caps() {
			global $wp_roles;
			if (class_exists('WP_Roles')) {
				if (!isset($wp_roles)) {
					$wp_roles = new WP_Roles();
				}
			}
			if (is_object($wp_roles)) {
				$this->set_reset_caps($wp_roles, 'add');
			}
		}
		/**
		 * Removes caps and roles
		 *
		 * @since WPAS 4.0
		 *
		 */
		public function remove_caps_roles() {
			global $wp_roles;
			if (class_exists('WP_Roles')) {
				if (!isset($wp_roles)) {
					$wp_roles = new WP_Roles();
				}
			}
			if (is_object($wp_roles)) {
				$this->set_reset_caps($wp_roles, 'remove');
			}
		}
		/**
		 * Set , reset capabilities
		 *
		 * @since WPAS 4.0
		 * @param object $wp_roles
		 * @param string $type
		 *
		 */
		public function set_reset_caps($wp_roles, $type) {
			$caps['enable'] = Array(
				'assign_kb_tags' => Array(
					'administrator'
				) ,
				'edit_kb_tags' => Array(
					'administrator'
				) ,
				'view_kc_com_dashboard' => Array(
					'administrator'
				) ,
				'edit_kb_group' => Array(
					'administrator'
				) ,
				'delete_kb_tags' => Array(
					'administrator'
				) ,
				'manage_kb_tags' => Array(
					'administrator'
				) ,
				'assign_kb_group' => Array(
					'administrator'
				) ,
				'edit_emd_panels' => Array(
					'administrator'
				) ,
				'delete_kb_group' => Array(
					'administrator'
				) ,
				'manage_kb_group' => Array(
					'administrator'
				) ,
			);
			$caps['enable'] = apply_filters('emd_ext_get_caps', $caps['enable'], $this->option_name);
			foreach ($caps as $stat => $role_caps) {
				foreach ($role_caps as $mycap => $roles) {
					foreach ($roles as $myrole) {
						if (($type == 'add' && $stat == 'enable') || ($stat == 'disable' && $type == 'remove')) {
							$wp_roles->add_cap($myrole, $mycap);
						} else if (($type == 'remove' && $stat == 'enable') || ($type == 'add' && $stat == 'disable')) {
							$wp_roles->remove_cap($myrole, $mycap);
						}
					}
				}
			}
		}
		/**
		 * Set app specific options
		 *
		 * @since WPAS 4.0
		 *
		 */
		private function set_options() {
			$access_views = Array();
			update_option($this->option_name . '_setup_pages', 1);
			//conf parameters for rating
			$has_ratings = Array(
				'panel_rating' => Array(
					'label' => 'Panel Rating',
					'lite' => 0,
					'entity' => 'emd_panel',
					'entity_label' => 'Panels',
					'mid_name' => 'emd_panel_info_emd_panel_0'
				)
			);
			update_option($this->option_name . '_has_ratings', $has_ratings);
			$ent_list = Array(
				'emd_panel' => Array(
					'label' => __('Panels', 'kc-com') ,
					'rewrite' => 'panels',
					'sortable' => 0,
					'searchable' => 1,
					'unique_keys' => Array(
						'emd_panel_id'
					) ,
					'req_blt' => Array(
						'blt_title' => Array(
							'msg' => __('Title', 'kc-com')
						) ,
					) ,
				) ,
			);
			update_option($this->option_name . '_ent_list', $ent_list);
			$shc_list['app'] = 'Knowledge Center';
			$shc_list['has_bs'] = 1;
			$shc_list['shcs']['std_panel'] = Array(
				"class_name" => "emd_panel",
				"type" => "std",
				'page_title' => __('Frequently Asked Questions', 'kc-com') ,
			);
			if (!empty($shc_list)) {
				update_option($this->option_name . '_shc_list', $shc_list);
			}
			$attr_list['emd_panel']['emd_panel_id'] = Array(
				'visible' => 1,
				'label' => __('ID', 'kc-com') ,
				'display_type' => 'hidden',
				'required' => 0,
				'srequired' => 0,
				'filterable' => 1,
				'list_visible' => 1,
				'mid' => 'emd_panel_info_emd_panel_0',
				'desc' => __('Unique identifier for a panel. It is incremented by 1.', 'kc-com') ,
				'autoinc_start' => 1,
				'autoinc_incr' => 1,
				'type' => 'char',
				'hidden_func' => 'autoinc',
				'uniqueAttr' => true,
			);
			$attr_list['emd_panel']['emd_panel_featured'] = Array(
				'visible' => 1,
				'label' => __('Featured', 'kc-com') ,
				'display_type' => 'checkbox',
				'required' => 0,
				'srequired' => 0,
				'filterable' => 1,
				'list_visible' => 1,
				'mid' => 'emd_panel_info_emd_panel_0',
				'type' => 'binary',
				'options' => array(
					1 => 1
				) ,
			);
			$attr_list['emd_panel']['emd_panel_color'] = Array(
				'visible' => 1,
				'label' => __('Context color', 'kc-com') ,
				'display_type' => 'select',
				'required' => 0,
				'srequired' => 0,
				'filterable' => 0,
				'list_visible' => 1,
				'mid' => 'emd_panel_info_emd_panel_0',
				'desc' => __('Colors the panel header to highlight or group similar content.', 'kc-com') ,
				'type' => 'char',
				'options' => array(
					'' => __('Please Select', 'kc-com') ,
					'default' => __('Default', 'kc-com') ,
					'primary' => __('Primary', 'kc-com') ,
					'success' => __('Success', 'kc-com') ,
					'info' => __('Info', 'kc-com') ,
					'danger' => __('Danger', 'kc-com') ,
					'warning' => __('Warning', 'kc-com')
				) ,
				'std' => 'default',
			);
			$attr_list['emd_panel']['emd_panel_initial_state'] = Array(
				'visible' => 1,
				'label' => __('Initial State', 'kc-com') ,
				'display_type' => 'radio',
				'required' => 0,
				'srequired' => 0,
				'filterable' => 0,
				'list_visible' => 1,
				'mid' => 'emd_panel_info_emd_panel_0',
				'desc' => __('Sets the initial state of a panel\'s content.', 'kc-com') ,
				'type' => 'char',
				'options' => array(
					'' => __('Hide content', 'kc-com') ,
					'in' => __('Show content', 'kc-com')
				) ,
			);
			$attr_list = apply_filters('emd_ext_attr_list', $attr_list, $this->option_name);
			if (!empty($attr_list)) {
				update_option($this->option_name . '_attr_list', $attr_list);
			}
			if (!empty($glob_forms_list)) {
				update_option($this->option_name . '_glob_forms_init_list', $glob_forms_list);
				if (get_option($this->option_name . '_glob_forms_list') === false) {
					update_option($this->option_name . '_glob_forms_list', $glob_forms_list);
				}
			}
			$tax_list['emd_panel']['kb_group'] = Array(
				'label' => __('KC Groups', 'kc-com') ,
				'default' => '',
				'type' => 'single',
				'hier' => 0,
				'sortable' => 0,
				'required' => 0,
				'srequired' => 0,
				'rewrite' => 'kb_group'
			);
			$tax_list['emd_panel']['kb_tags'] = Array(
				'label' => __('KC Tags', 'kc-com') ,
				'default' => '',
				'type' => 'multi',
				'hier' => 0,
				'sortable' => 0,
				'required' => 0,
				'srequired' => 0,
				'rewrite' => 'kb_tags'
			);
			if (!empty($tax_list)) {
				update_option($this->option_name . '_tax_list', $tax_list);
			}
			$emd_activated_plugins = get_option('emd_activated_plugins');
			if (!$emd_activated_plugins) {
				update_option('emd_activated_plugins', Array(
					'kc-com'
				));
			} elseif (!in_array('kc-com', $emd_activated_plugins)) {
				array_push($emd_activated_plugins, 'kc-com');
				update_option('emd_activated_plugins', $emd_activated_plugins);
			}
			//conf parameters for incoming email
			//conf parameters for inline entity
			//conf parameters for calendar
			//action to configure different extension conf parameters for this plugin
			do_action('emd_ext_set_conf', 'kc-com');
		}
		/**
		 * Reset app specific options
		 *
		 * @since WPAS 4.0
		 *
		 */
		private function reset_options() {
			delete_option($this->option_name . '_ent_list');
			delete_option($this->option_name . '_shc_list');
			delete_option($this->option_name . '_attr_list');
			delete_option($this->option_name . '_tax_list');
			delete_option($this->option_name . '_rel_list');
			delete_option($this->option_name . '_adm_notice1');
			delete_option($this->option_name . '_adm_notice2');
			delete_option($this->option_name . '_setup_pages');
			$emd_activated_plugins = get_option('emd_activated_plugins');
			if (!empty($emd_activated_plugins)) {
				$emd_activated_plugins = array_diff($emd_activated_plugins, Array(
					'kc-com'
				));
				update_option('emd_activated_plugins', $emd_activated_plugins);
			}
			delete_option($this->option_name . '_has_ratings');
		}
		/**
		 * Show install notices
		 *
		 * @since WPAS 4.0
		 *
		 * @return html
		 */
		public function install_notice() {
			if (isset($_GET[$this->option_name . '_adm_notice1'])) {
				update_option($this->option_name . '_adm_notice1', true);
			}
			if (current_user_can('manage_options') && get_option($this->option_name . '_adm_notice1') != 1) {
?>
<div class="updated">
<?php
				printf('<p><a href="%1s" target="_blank"> %2$s </a>%3$s<a style="float:right;" href="%4$s"><span class="dashicons dashicons-dismiss" style="font-size:15px;"></span>%5$s</a></p>', 'https://docs.emdplugins.com/docs/knowledge-center-community-documentation/?pk_campaign=kc-com&pk_source=plugin&pk_medium=link&pk_content=notice', __('New To Knowledge Center? Review the documentation!', 'wpas') , __('&#187;', 'wpas') , esc_url(add_query_arg($this->option_name . '_adm_notice1', true)) , __('Dismiss', 'wpas'));
?>
</div>
<?php
			}
			if (isset($_GET[$this->option_name . '_adm_notice2'])) {
				update_option($this->option_name . '_adm_notice2', true);
			}
			if (current_user_can('manage_options') && get_option($this->option_name . '_adm_notice2') != 1) {
?>
<div class="updated">
<?php
				printf('<p><a href="%1s" target="_blank"> %2$s </a>%3$s<a style="float:right;" href="%4$s"><span class="dashicons dashicons-dismiss" style="font-size:15px;"></span>%5$s</a></p>', 'https://emdplugins.com/plugin_tag/kc-com/?pk_campaign=kc-com&pk_source=plugin&pk_medium=link&pk_content=notice', __('Upgrade Now to Premium Editions!', 'wpas') , __('&#187;', 'wpas') , esc_url(add_query_arg($this->option_name . '_adm_notice2', true)) , __('Dismiss', 'wpas'));
?>
</div>
<?php
			}
			if (current_user_can('manage_options') && get_option($this->option_name . '_setup_pages') == 1) {
				echo "<div id=\"message\" class=\"updated\"><p><strong>" . __('Welcome to Knowledge Center', 'kc-com') . "</strong></p>
           <p class=\"submit\"><a href=\"" . add_query_arg('setup_kc_com_pages', 'true', admin_url('index.php')) . "\" class=\"button-primary\">" . __('Setup Knowledge Center Pages', 'kc-com') . "</a> <a class=\"skip button-primary\" href=\"" . add_query_arg('skip_setup_kc_com_pages', 'true', admin_url('index.php')) . "\">" . __('Skip setup', 'kc-com') . "</a></p>
         </div>";
			}
		}
		/**
		 * Setup pages for components and redirect to dashboard
		 *
		 * @since WPAS 4.0
		 *
		 */
		public function setup_pages() {
			if (!is_admin()) {
				return;
			}
			global $wpdb;
			if (!empty($_GET['setup_' . $this->option_name . '_pages'])) {
				$shc_list = get_option($this->option_name . '_shc_list');
				$shc_list = apply_filters('emd_ext_chart_list', $shc_list, $this->option_name);
				$types = Array(
					'forms',
					'charts',
					'shcs',
					'datagrids',
					'integrations'
				);
				foreach ($types as $shc_type) {
					if (!empty($shc_list[$shc_type])) {
						foreach ($shc_list[$shc_type] as $keyshc => $myshc) {
							if (isset($myshc['page_title'])) {
								$pages[$keyshc] = $myshc;
							}
						}
					}
				}
				foreach ($pages as $key => $page) {
					$found = "";
					$page_content = "[" . $key . "]";
					$found = $wpdb->get_var($wpdb->prepare("SELECT ID FROM " . $wpdb->posts . " WHERE post_type='page' AND post_content LIKE %s LIMIT 1;", "%{$page_content}%"));
					if ($found != "") {
						continue;
					}
					$page_data = array(
						'post_status' => 'publish',
						'post_type' => 'page',
						'post_author' => get_current_user_id() ,
						'post_title' => $page['page_title'],
						'post_content' => $page_content,
						'comment_status' => 'closed'
					);
					$page_id = wp_insert_post($page_data);
				}
				delete_option($this->option_name . '_setup_pages');
				wp_redirect(admin_url('index.php?kc-com-installed=true'));
				exit;
			}
			if (!empty($_GET['skip_setup_' . $this->option_name . '_pages'])) {
				delete_option($this->option_name . '_setup_pages');
				wp_redirect(admin_url('index.php?'));
				exit;
			}
		}
		public function tinymce_fix($init) {
			$init['wpautop'] = false;
			return $init;
		}
	}
endif;
return new Kc_Com_Install_Deactivate();
